window.Gleam.initSearch([{"doc":"lamb","title":"lamb","content":"# lamb\n\n[![Package Version](https://img.shields.io/hexpm/v/lamb)](https://hex.pm/packages/lamb)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/lamb/)\n\nA gleam library for operating and querying ETS tables.\n\n```gleam\nimport lamb\n\npub fn main() {\n  let assert Ok(table) = lamb.create_table(Private, &quot;test_table&quot;)\n\n  lamb.insert(table, &quot;a&quot;, 1)\n  lamb.insert(table, &quot;b&quot;, 2)\n  lamb.insert(table, &quot;c&quot;, 3)\n  lamb.insert(table, &quot;d&quot;, 4)\n  lamb.insert(table, &quot;e&quot;, 5)\n\n  let assert Records([_, _] as a, step) = lamb.partial(table, by: 2)\n  let assert Records([_, _] as b, step) = lamb.continue(step)\n  let assert End([_] as c) = lamb.continue(step)\n\n  lamb.delete_table(table)\n}\n```\n\nFurther documentation can be found at &lt;https://hexdocs.pm/lamb&gt;.\n\n## Development\n\n```sh\ngleam run   # Run the project\ngleam test  # Run the tests\n```\n\n---\n\nWork in progress notes.\n\n# Table API\n\nCurrently viewing the table creating in terms of protection levels:\n  - `Private`, private table, not named.\n  - `Protected`, protected table, named.\n  - `Public`, public table, named.\n\nIf we would like more precision than this, maybe having an `options` helper with default options would\nhelp to tinker with tables:\n\n```gleam\npub fn options() -&gt; Options {\n  https://www.erlang.org/doc/apps/stdlib/ets.html#info/1\n  todo\n}\n```\n\nStill need to figure out what is going to be the API for differentiating between `set` and `bag` tables.\n\n# Query API\n\nMatchspecs are composed by a Tuple of arity 3 called a `MatchFunction`:\n\n- A `Head` that contains the shape of the data we want to match to, as well as variable declarations.\n- A list of `Condition` expressions that can help filter data through predicates.\n- A `Body` that declares the shape and variables we&#39;d like to output from the `MatchFunction`.\n\n```erlang\n{Head, [Condition], [Body]}\n```\n\nOperating on these 3 pieces may be tackled in different ways.\n\n## Alternative 1\n\nHave an API that composes matchspecs together with a builder pattern:\n\n```gleam\nlet query =\n  q.new()                                  // Builds a basic default match function.\n  |&gt; q.bind(Person)                        // Modifies the head to match the constructor.\n  |&gt; q.match(field: 5, &quot;Citizen&quot;)          // Modifies the head to match the value.\n  |&gt; q.where(field: 3, op: &quot;&gt;=&quot;, than: 18) // Adds a condition expression.\n\nquery |&gt; q.select(object())                // Returns the whole object #(index, record).\nquery |&gt; q.select(index())                 // Returns just the index of record.\nquery |&gt; q.select(#(v(2), v(1)))           // Returns the 2nd and 1st variables in a tuple.\nquery |&gt; q.select(False)                   // Returns false for each matching record.\nquery |&gt; q.select(Driver)                  // Maps the variables to a constructor.\nquery |&gt; q.select(fn(last, first) {        // Alternative mapper to above.\n  Driver(last, first)\n})\n```\n\n## Alternative 2\n\nBuild a &quot;nice&quot; parser to compose complex queries:\n\n```gleam\n\nlet query = &quot;\n  from Person\n  select 2, 1\n  where &#39;Citizen&#39; = 5\n  where 3 &gt;= 18\n  select #(0, 1, 2)\n&quot;\n```\n\n## Alternative 3\n\nJust build a more straightforward mapper for matchspecs.\n\n```gleam\nlet head = #(any(), record(&quot;Person&quot;, v(1), v(2), v(3), &quot;Citizen&quot;))\nlet condition = #(&quot;&gt;=&quot;, v(3), 18)\nlet body = body(record(&quot;Driver&quot;, v(2), v(1)))\nlet query_driver_elegibility = #(head, [condition], [body])\nlamb.search(table, [query_driver_elegibility])\n```\n\n## Notes on types and validation\n\nNone of these alternatives currently provide a good way of enforcing types but are meant to fail gracefully if\nthere are errors. Querying so far is a &quot;dynamic&quot; operation.\n\nAn alternative would be to run validations at runtime with the help of a schema but that would\nbe quite a big lift for my current purposes.\n\nMaybe another way is to provide a validator at init time, but this would exclusively check the validity of the\nmatchspecs.\n\n# Checkout these ETS APIs\n\n* https://www.erlang.org/doc/apps/stdlib/ets.html#select_delete/2\n* https://www.erlang.org/doc/apps/stdlib/ets.html#fun2ms/1\n* https://www.erlang.org/doc/apps/stdlib/ets.html#test_ms/2\n* https://www.erlang.org/doc/apps/stdlib/ets.html#table/2\n* https://www.erlang.org/doc/apps/stdlib/qlc\n* https://www.erlang.org/doc/apps/stdlib/erl_parse.html#abstract/1\n","url":"index.html"},{"doc":"lamb","title":"Access","content":"pub type Access {\n  Public\n  Protected\n  Private\n}\n\nPublic\n\n\nProtected\n\n\nPrivate\n\n\nSynonyms:\nlamb.Access\nlamb Access","url":"lamb.html#Access"},{"doc":"lamb","title":"MatchFunction","content":"pub type MatchFunction\n\n\nSynonyms:\nlamb.MatchFunction\nlamb MatchFunction","url":"lamb.html#MatchFunction"},{"doc":"lamb","title":"Partial","content":"pub type Partial(record) {\n  Records(List(record), Step)\n  End(List(record))\n}\n\nRecords(List(record), Step)\n\n\nEnd(List(record))\n\n\nSynonyms:\nlamb.Partial\nlamb Partial","url":"lamb.html#Partial"},{"doc":"lamb","title":"Step","content":"pub type Step\n\n\nSynonyms:\nlamb.Step\nlamb Step","url":"lamb.html#Step"},{"doc":"lamb","title":"Table","content":"pub opaque type Table(index, record)\n\n\nSynonyms:\nlamb.Table\nlamb Table","url":"lamb.html#Table"},{"doc":"lamb","title":"all","content":"pub fn all(table: Table(a, b)) -&gt; List(b)\n\nSynonyms:\nlamb.all\nlamb all","url":"lamb.html#all"},{"doc":"lamb","title":"continue","content":"pub fn continue(step: Step) -&gt; Partial(a)\n\nSynonyms:\nlamb.continue\nlamb continue","url":"lamb.html#continue"},{"doc":"lamb","title":"count","content":"pub fn count(table: Table(a, b)) -&gt; Int\n\nSynonyms:\nlamb.count\nlamb count","url":"lamb.html#count"},{"doc":"lamb","title":"create_table","content":"pub fn create_table(\n  access: Access,\n  name: String,\n) -&gt; Result(Table(a, b), Nil)\n\nSynonyms:\nlamb.create_table\nlamb create_table","url":"lamb.html#create_table"},{"doc":"lamb","title":"delete","content":"pub fn delete(table: Table(a, b), index: a) -&gt; Nil\n\nSynonyms:\nlamb.delete\nlamb delete","url":"lamb.html#delete"},{"doc":"lamb","title":"delete_table","content":"pub fn delete_table(table: Table(a, b)) -&gt; Nil\n\nSynonyms:\nlamb.delete_table\nlamb delete_table","url":"lamb.html#delete_table"},{"doc":"lamb","title":"from_name","content":"pub fn from_name(name: String) -&gt; Result(Table(a, b), Nil)\n\nSynonyms:\nlamb.from_name\nlamb from_name","url":"lamb.html#from_name"},{"doc":"lamb","title":"get","content":"pub fn get(table: Table(a, b), index: a) -&gt; Result(b, Nil)\n\nSynonyms:\nlamb.get\nlamb get","url":"lamb.html#get"},{"doc":"lamb","title":"insert","content":"pub fn insert(table: Table(a, b), index: a, record: b) -&gt; Nil\n\nSynonyms:\nlamb.insert\nlamb insert","url":"lamb.html#insert"},{"doc":"lamb","title":"is_alive","content":"pub fn is_alive(table: Table(a, b)) -&gt; Bool\n\nSynonyms:\nlamb.is_alive\nlamb is_alive","url":"lamb.html#is_alive"},{"doc":"lamb","title":"partial","content":"pub fn partial(table: Table(a, b), by limit: Int) -&gt; Partial(b)\n\nSynonyms:\nlamb.partial\nlamb partial","url":"lamb.html#partial"},{"doc":"lamb","title":"search","content":"pub fn search(\n  table: Table(a, b),\n  where expression: List(MatchFunction),\n) -&gt; List(c)\n\nSynonyms:\nlamb.search\nlamb search","url":"lamb.html#search"},{"doc":"lamb","title":"search_count","content":"pub fn search_count(\n  table: Table(a, b),\n  where expression: List(MatchFunction),\n) -&gt; Int\n\nSynonyms:\nlamb.search_count\nlamb search_count","url":"lamb.html#search_count"},{"doc":"lamb","title":"search_partial","content":"pub fn search_partial(\n  table: Table(a, b),\n  by limit: Int,\n  where expression: List(MatchFunction),\n) -&gt; Partial(c)\n\nSynonyms:\nlamb.search_partial\nlamb search_partial","url":"lamb.html#search_partial"},{"doc":"lamb","title":"lamb","content":"","url":"lamb.html"},{"doc":"lamb/query","title":"lamb/query","content":"","url":"lamb/query.html"}]);